import { generateSlug } from 'random-word-slugs';
import { Surreal } from 'surrealdb.js';
import { Entry } from '~/entries/interfaces/entry.type.js';

class Database {
	private readonly _db: Surreal;

	constructor() {
		this._db = new Surreal();
	}

	get db() {
		return this._db;
	}

	async init() {
		try {
			// Connect to the database
			await this.db.connect('ws://127.0.0.1:8000');
	
			// Signin as a namespace, database, or root user
			await this.db.signin({
				username: 'root',
				password: 'root',
			});
	
			// Select a specific namespace / database
			await this.db.use({
				namespace: 'surreal',
				database: 'entries'
			});

			// Define stuff
			await this.db.query(`
			DEFINE TABLE entries SCHEMAFULL;
			
			DEFINE FIELD slug ON TABLE entries FLEXIBLE TYPE string;
			DEFINE INDEX slug_index ON TABLE entries COLUMNS slug UNIQUE;
			
			DEFINE FIELD expires_at ON TABLE entries TYPE datetime
			DEFAULT time::now() + duration::from::days(7);
			
			DEFINE FIELD data ON TABLE entries FLEXIBLE TYPE object
			DEFAULT {};

			DEFINE EVENT updated ON TABLE entries WHEN $before.data != $after.data THEN (
				UPDATE entries SET expires_at = (time::now() + duration::from::days(7)) WHERE slug = $after.slug
			);
			`);
		}
		catch (error) {
			throw error;
		}
	}

	async getEntry(slug: string): Promise<Entry> {
		const [results] = await this.db.query<Entry[]>('SELECT * OMIT id FROM entries WHERE slug = $slug', { slug });
		return results[0];
	}

	async createEntry(): Promise<Entry> {
		const slug = generateSlug();

		const [result] = await this.db.create<Entry>('entries', {
			slug
		});

		delete result.id;

		return result;
	}

	async updateEntry(slug: string, data: JSON): Promise<Entry> {
		if (!await this.getEntry(slug)) {
			return null;
		}

		await this.db.query('UPDATE entries SET data = $data WHERE slug = $slug', {
			data,
			slug
		});

		return this.getEntry(slug);
	}

	async deleteEntry(slug: string): Promise<boolean> {
		if (!await this.getEntry(slug)) {
			return false;
		}

		await this.db.query('DELETE FROM entries WHERE slug = $slug', {
			slug
		});

		return true;
	}
}

export default new Database();